
! gfortran -Wall -fcheck=all -o strails fitsio.f95 strails.f08 -lcfitsio

program strails

  use fitsio

  implicit none

  integer :: status, n
  integer, dimension(3) :: naxes = [ 3465, 5202, 3 ]
  real, allocatable, dimension(:,:,:) :: image, out1, out2, out
  real, allocatable, dimension(:,:) :: L,u,v,L1,u1,v1,L2,u2,v2
  character(len=666) :: filename
  real :: w
  integer :: i

  allocate(image(naxes(1),naxes(2),naxes(3)),out(naxes(1),naxes(2),naxes(3)),&
       out1(naxes(1),naxes(2),naxes(3)),out2(naxes(1),naxes(2),naxes(3)))
  allocate(L(naxes(1),naxes(2)), u(naxes(1),naxes(2)), v(naxes(1),naxes(2)),&
       L1(naxes(1),naxes(2)), u1(naxes(1),naxes(2)), v1(naxes(1),naxes(2)), &
       L2(naxes(1),naxes(2)), u2(naxes(1),naxes(2)), v2(naxes(1),naxes(2)))
  out1 = 0
  out2 = 0
  status = 0
  n = 0

  do
     read(*,'(a)',end=99) filename
     write(*,*) trim(filename),':'
     call fits_open_image(25,filename,READONLY,status)
     call fits_read_3d(25,image,status)
     call fits_close_file(25,status)
     if( status /= 0 ) exit
     n = n + 1

     where( image > out1 )
        out1 = image
     end where

     out2 = out2 + image

  end do
99 continue

  out2 = out2 / n

  call cieluv(out1(:,:,1),out1(:,:,2),out1(:,:,3),L1,u1,v1)
  call cieluv(out2(:,:,1),out2(:,:,2),out2(:,:,3),L2,u2,v2)
  do i = 1, naxes(2)
     w = 1 / (1 + exp(-(i-1000.0)/200.0))
     L(:,i) = L1(:,i)*w + (1-w)*L2(:,i)
     u(:,i) = u1(:,i)*w + (1-w)*u2(:,i)
     v(:,i) = v1(:,i)*w + (1-w)*v2(:,i)
  end do
  call cieluv1(L,u,v,out(:,:,1),out(:,:,2),out(:,:,3))

  call fits_create_file(26,'strails.fits',status)
  call fits_insert_img(26,-32,3,naxes,status)
  call fits_write_key(26,'CSPACE','CIE 1931 XYZ','',status)
  call fits_write_3d(26,out,status)
  call fits_close_file(26,status)
  call fits_report_error('STDERR',status)

  call fits_create_file(26,'strails1.fits',status)
  call fits_insert_img(26,-32,3,naxes,status)
  call fits_write_key(26,'CSPACE','CIE 1931 XYZ','',status)
  call fits_write_3d(26,out1,status)
  call fits_close_file(26,status)
  call fits_report_error('STDERR',status)

  call fits_create_file(26,'strails2.fits',status)
  call fits_insert_img(26,-32,3,naxes,status)
  call fits_write_key(26,'CSPACE','CIE 1931 XYZ','',status)
  call fits_write_3d(26,out2,status)
  call fits_close_file(26,status)
  call fits_report_error('STDERR',status)

  do i = 1, naxes(2)
     w = 0.5 + 0.5 / (1 + exp(-(i-1000.0)/200.0))
     L(:,i) = L1(:,i)*w
     u(:,i) = u1(:,i)*w
     v(:,i) = v1(:,i)*w
  end do
  call cieluv1(L,u,v,out(:,:,1),out(:,:,2),out(:,:,3))

  call fits_create_file(26,'strails3.fits',status)
  call fits_insert_img(26,-32,3,naxes,status)
  call fits_write_key(26,'CSPACE','CIE 1931 XYZ','',status)
  call fits_write_3d(26,out,status)
  call fits_close_file(26,status)
  call fits_report_error('STDERR',status)

contains

  subroutine cieluv(X,Y,Z,L,u,v)

    real, dimension(:,:), intent(in) :: X,Y,Z
    real, dimension(:,:), intent(out) :: L,u,v
    real :: Y0, c

    Y0 = 65536
    c = (6.0 / 29.0)**3

    where( Y / Y0 <= c )
       L = (29.0 / 3.0)**2 * Y / Y0
    elsewhere
       L = 116*(Y / Y0)**(1.0/3.0) - 16
    end where
    u = 4*X / (X + 15*Y + 3*Z)
    v = 9*Y / (X + 15*Y + 3*Z)
    u = 13*L*(u - 0.2009)
    v = 13*L*(v - 0.4610)

  end subroutine cieluv

  subroutine cieluv1(L,u,v,X,Y,Z)

    real, dimension(:,:), intent(in) :: L,u,v
    real, dimension(:,:), intent(out) :: X,Y,Z
    real, dimension(size(u,1),size(u,2)) :: u1,v1
    real :: Y0

    Y0 = 65536
    u1 = u / (13 * L) + 0.2009
    v1 = v / (13 * L) + 0.4610

    where( L <= 8 )
       Y = Y0 * L * (3.0 / 29.0)**3
    elsewhere
       Y = Y0 * ((L + 16)/116)**3
    end where
    X = Y * (9 * u1) / (4 * v1)
    Z = Y * (12 - 3*u1 - 20*v1) / (4*v1)

  end subroutine cieluv1



  subroutine fits_read_3d(unit,image,status)

    real, parameter :: nullval = 0.0
    integer, parameter :: group = 0

    integer, intent(in) :: unit
    real, dimension(:,:,:), intent(out) :: image
    integer, intent(in out) :: status

    integer :: dim1, dim2, naxes1, naxes2, naxes3
    logical :: anyf

    dim1 = size(image,1)
    dim2 = size(image,2)
    naxes1 = size(image,1)
    naxes2 = size(image,2)
    naxes3 = size(image,3)

    call ftg3de(unit,group,nullval,dim1,dim2,naxes1,naxes2,naxes3,image,anyf,status)

  end subroutine fits_read_3d


  subroutine fits_write_3d(unit,image,status)

    integer, parameter :: group = 0

    integer, intent(in) :: unit
    real, dimension(:,:,:), intent(in) :: image
    integer, intent(in out) :: status

    integer :: dim1, dim2, naxes1, naxes2, naxes3

    dim1 = size(image,1)
    dim2 = size(image,2)
    naxes1 = size(image,1)
    naxes2 = size(image,2)
    naxes3 = size(image,3)

    call ftp3de(unit,group,dim1,dim2,naxes1,naxes2,naxes3,image,status)

  end subroutine fits_write_3d


end program strails
